@@ -3,19 +3,22 @@
  *  Copyright (C) 2002-8 Simon Urbanek
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; version 2 of the License
+ *  it under the terms of the GNU Lesser General Public License as published
+ *  by the Free Software Foundation; version 2.1 of the License
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  GNU Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
+ *  You should have received a copy of the GNU Lesser General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- *  $Id: Rsrv.h 234 2008-07-29 13:59:42Z urbanek $
+ *  Note: This header file is licensed under LGPL to allow other
+ *        programs to use it under LGPL. Rserve itself is licensed under GPL.
+ *
+ *  $Id: Rsrv.h 265 2009-01-28 20:58:14Z urbanek $
  */
 
 /* external defines:
@@ -29,7 +32,7 @@
 #include "config.h"
 #endif
 
-#define RSRV_VER 0x000502 /* Rserve v0.5-2 */
+#define RSRV_VER 0x000600 /* Rserve v0.6-0 */
 
 #define default_Rsrv_port 6311
 
@@ -160,6 +163,9 @@ struct phdr { /* always 16 bytes */
 #define ERR_out_of_mem       0x4d /* out of memory. the connection is usually
 									 closed after this error was sent */
 
+/* since 0.6-0 */
+#define ERR_ctrl_closed      0x4e /* control pipe to the master process is closed or broken */
+
 /* since 0.4-0 */
 #define ERR_session_busy     0x50 /* session is still busy */
 #define ERR_detach_failed    0x51 /* unable to detach seesion (cannot determine
@@ -193,6 +199,20 @@ struct phdr { /* always 16 bytes */
 #define CMD_detachedVoidEval 0x031 /* string : session key; doesn't */
 #define CMD_attachSession    0x032 /* session key : - */  
 
+/* control commands (since 0.6-0) - passed on to the master process */
+/* Note: currently all control commands are asychronous, i.e. RESP_OK
+   indicates that the command was enqueued in the master pipe, but there
+   is no guarantee that it will be processed. Moreover non-forked
+   connections (e.g. the default debug setup) don't process any
+   control commands until the current client connection is closed so
+   the connection issuing the control command will never see its
+   result.
+*/
+#define CMD_ctrl            0x40  /* -- not a command - just a constant -- */
+#define CMD_ctrlEval        0x42  /* string : - */
+#define CMD_ctrlSource      0x45  /* string : - */
+#define CMD_ctrlShutdown    0x44  /* - : - */
+
 /* 'internal' commands (since 0.1-9) */
 #define CMD_setBufferSize 0x081  /* [int sendBufSize] 
 				  this commad allow clients to request
@@ -200,6 +220,7 @@ struct phdr { /* always 16 bytes */
 				  transported from Rserve to the client.
 				  (incoming buffer is resized automatically)
 				 */
+#define CMD_setEncoding   0x082  /* string (one of "native","latin1","utf8") : -; since 0.5-3 */
 
 /* special commands - the payload of packages with this mask does not contain defined parameters */
 
