@@ -1,22 +1,25 @@
 /*
  *  C++ Interface to Rserve
- *  Copyright (C) 2004 Simon Urbanek, All rights reserved.
+ *  Copyright (C) 2004-8 Simon Urbanek, All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  it under the terms of the GNU Lesser General Public License as published by
+ *  the Free Software Foundation; version 2.1 of the License
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  GNU Leser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
+ *  You should have received a copy of the GNU Lesser General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- *  $Id: Rconnection.cc 236 2008-08-17 22:01:01Z urbanek $
+ *  Although this code is licensed under LGPL v2.1, we strongly encourage
+ *  everyone modifying this software to contribute back any improvements and
+ *  bugfixes to the project for the benefit all other users. Thank you.
+ * 
+ *  $Id: Rconnection.cc 265 2009-01-28 20:58:14Z urbanek $
  */
 
 /* external defines:
@@ -24,6 +27,20 @@
    see also SOCK_ERROR, MAIN and other defines in sisocks.h
 */
 
+/* locally generated status error and return codes:
+   -1  - operation failed (e.g. connect failed)
+   -2  - handhake failed
+   -3  - invalid ID string
+   -4  - protocol not supported
+   -5  - not connected
+   -6  - - unused -
+   -7  - remote connection close
+   -8  - malformed packet
+   -9  - send error
+   -10 - out of memory
+   -11 - operation is unsupported (e.g. unix login while crypt is not linked)
+   -12 - eval didn't return a SEXP (possibly the server is too old/buggy or crashed)
+ */
 #include "Rconnection.h"
 
 #include <stdio.h>
@@ -50,7 +67,7 @@
 #endif
 
 // NOTE: 0103 compatibility has not been established! use at your own risk!
-static char *myID= "Rsrv0103QAP1"; /* this client supports up to protocol version 0103 */
+static const char *myID = "Rsrv0103QAP1"; /* this client supports up to protocol version 0103 */
 
 static Rexp *new_parsed_Rexp(unsigned int *d, Rmessage *msg) {
     int type=ptoi(*d)&0x3f;
@@ -93,7 +110,7 @@ Rmessage::Rmessage() {
 
 Rmessage::Rmessage(int cmd) {
     memset(&head,0,sizeof(head));
-    head.cmd=cmd&0x3f;
+    head.cmd = cmd;
     data=0;
     len=0;
     complete=1;
@@ -105,7 +122,7 @@ Rmessage::Rmessage(int cmd, const char *txt) {
     if ((tl&3)>0)
         tl=(tl+4)&0xffffc; // allign the text
     len=tl+4; // message length is tl + 4 (short format only)
-    head.cmd=cmd&0x3f;
+    head.cmd=cmd;
     head.len=len;
     data=(char*)malloc(tl+16);
     memset(data,0,tl+16);
@@ -117,7 +134,7 @@ Rmessage::Rmessage(int cmd, const char *txt) {
 Rmessage::Rmessage(int cmd, const void *buf, int dlen, int raw_data) {
     memset(&head,0,sizeof(head));
     len=(raw_data)?dlen:(dlen+4);
-    head.cmd=cmd&0x3f;
+    head.cmd=cmd;
     head.len=len;
     data=(char*)malloc(len);
     memcpy(data, (raw_data)?buf:((char*)buf+4), dlen);
@@ -129,7 +146,7 @@ Rmessage::Rmessage(int cmd, const void *buf, int dlen, int raw_data) {
 Rmessage::Rmessage(int cmd, int i) {
     memset(&head,0,sizeof(head));
     len=8; // DT_INT+len (4) + payload-1xINT (4)
-    head.cmd=cmd&0x3f;
+    head.cmd=cmd;
     head.len=len;
     data=(char*)malloc(8);
     *((int*)data)=itop(SET_PAR(DT_INT,4));
@@ -255,7 +272,11 @@ Rexp::Rexp(unsigned int *pos, Rmessage *msg) {
 Rexp::Rexp(int type, const char *data, int len, Rexp *attr) {
     this->attr=attr; master=this; rcount=0; attribs=0;
     this->type=type;
+    this->msg=0;
     if (len>0) {
+#ifdef DEBUG_CXX
+        fprintf(stderr, "Rexp::Rexp %p: allocating %d bytes\n", this, len);
+#endif
         this->data=(char*) malloc(len);
         memcpy(this->data, data, len);
         this->len=len;
@@ -334,7 +355,7 @@ Rexp *Rexp::attribute(const char *name) {
     return (attr && attr->type==XT_LIST)?((Rlist*)attr)->entryByTagName(name):0;
 }
 
-char **Rexp::attributeNames() {
+const char **Rexp::attributeNames() {
     if (!attr || attr->type!=XT_LIST)
         return 0;
     if (attribs==0) {
@@ -344,7 +365,7 @@ char **Rexp::attributeNames() {
             if (l->tag && l->tag->type==XT_SYM) attribs++;
             l=l->tail;
         }
-        attrnames=(char**) malloc(sizeof(char*)*(attribs+1));
+        attrnames=(const char**) malloc(sizeof(char*)*(attribs+1));
         l = (Rlist*) attr;
         while (l && l->type==XT_LIST) {
             if (l->tag && l->tag->type==XT_SYM)
@@ -707,8 +728,9 @@ int Rconnection::assign(const char *symbol, Rexp *exp) {
         delete(msg);
         return res;
     }
-    // we should check response code here ...
-    return 0;
+    if (!res) res = CMD_STAT(msg->command());
+    delete(msg);
+    return res;
 }
 
 int Rconnection::voidEval(const char *cmd) {
@@ -717,7 +739,7 @@ int Rconnection::voidEval(const char *cmd) {
     return status;
 }
     
-Rexp *Rconnection::eval(const char *cmd, int *status, int opt) {
+Rexp *Rconnection::eval(const char *cmd, int *status, int opt) { /* opt = 1 -> void eval */
     Rmessage *msg=new Rmessage();
     Rmessage *cmdMessage=new Rmessage((opt&1)?CMD_voidEval:CMD_eval, cmd);
     int res=request(msg,cmdMessage);
@@ -727,9 +749,9 @@ Rexp *Rconnection::eval(const char *cmd, int *status, int opt) {
         delete(msg);
         return 0;
     }
-    if (!res && (msg->pars!=1 || (ptoi(msg->par[0][0])&0x3f)!=DT_SEXP)) {
+    if ((opt & 1) == 0 && !res && (msg->pars!=1 || (ptoi(msg->par[0][0])&0x3f)!=DT_SEXP)) {
         delete(msg);
-        if (status) *status=-10; // returned object is not SEXP
+        if (status) *status=-12; // returned object is not SEXP
         return 0;
     }
     if (res) {
@@ -846,3 +868,42 @@ int Rconnection::login(const char *user, const char *pwd) {
   free(authbuf);
   return res;
 }
+
+#ifdef CMD_ctrl
+
+/* server control methods */
+int serverEval(const char *cmd);
+int serverSource(const char *fn);
+int serverShutdown();
+
+int Rconnection::serverEval(const char *cmd) {
+    Rmessage *msg = new Rmessage(); /* result message */
+    Rmessage *cmdMessage = new Rmessage(CMD_ctrlEval, cmd); /* request message */
+    int res = request(msg, cmdMessage);
+    delete (cmdMessage);
+    if (!res) res = CMD_STAT(msg->command());
+    delete (msg);
+    return res;
+}
+
+int Rconnection::serverSource(const char *fn) {
+    Rmessage *msg = new Rmessage(); /* result message */
+    Rmessage *cmdMessage = new Rmessage(CMD_ctrlSource, fn); /* request message */
+    int res = request(msg, cmdMessage);
+    delete (cmdMessage);
+    if (!res) res = CMD_STAT(msg->command());
+    delete (msg);
+    return res;
+}
+
+int Rconnection::serverShutdown() {
+    Rmessage *msg = new Rmessage(); /* result message */
+    Rmessage *cmdMessage = new Rmessage(CMD_ctrlShutdown); /* request message */
+    int res = request(msg, cmdMessage);
+    delete (cmdMessage);
+    if (!res) res = CMD_STAT(msg->command());
+    delete (msg);
+    return res;
+}
+
+#endif
